"""
Auxiliary functions for eniric
"""
import collections
import errno
import os
from typing import Any, List, Optional, Sequence, Tuple, Union

import astropy.constants as const
import numpy as np
from numpy import ndarray

import eniric

c = const.c
# Band limits.
bands_ = {
    "VIS": (0.38, 0.78),
    "GAP": (0.78, 0.83),
    "Z": (0.83, 0.93),
    "Y": (1.0, 1.1),
    "J": (1.17, 1.33),
    "H": (1.5, 1.75),
    "K": (2.07, 2.35),
    "CONT": (0.45, 1.05),
    "NIR": (0.83, 2.35),
}

bands_.update(eniric.custom_bands)


def band_selector(wav: ndarray, flux: ndarray, band: str) -> Tuple[ndarray, ndarray]:
    """Select a specific wavelength band.

    Parameters
    ----------
    wav: array-like
        Wavelength values.
    flux: array-like
        Flux values.
    band: str
        Band letter to select, upper or lower case is supported. Options
        are ("ALL" or ""), "VIS", "GAP", "Z", "Y", "J", "H", "K".
    """
    band = band.upper()

    if band in ["ALL", ""]:
        return wav, flux
    else:
        band_min, band_max = band_limits(band)
        return wav_selector(wav, flux, band_min, band_max)


def band_limits(band: str) -> Tuple[float, float]:
    """ Get wavelength limits of band in microns.

    Parameters
    ----------
    band: str
        Band letter to get wavelength range for.

    Returns
    -------
    wav_min: float
        Lower wavelength bound of band in microns
    wav_max: float
        Upper wavelength bound of band in microns
    """
    if not isinstance(band, str):
        raise AttributeError("Band name must be a string")
    else:
        band = band.upper()

    if band in bands_:
        return bands_[band]
    else:
        raise ValueError("The band {0} requested is not a valid option".format(band))


def band_middle(band: str) -> float:
    """Calculate band mid-point.

    Input
    -----
    band: str
        Band label

    Return
    -------
    middle: float
        Wavelength at middle band.
    """
    band_min, band_max = band_limits(band)

    return (band_min + band_max) / 2


def wav_selector(
    wav: Union[ndarray, List[float]],
    flux: Union[ndarray, List[float]],
    wav_min: float,
    wav_max: float,
) -> Tuple[ndarray, ndarray]:
    """
    function that returns wavelength and flux within a giving range

    Parameters
    ----------
    wav: array-like
        Wavelength array.
    flux: array-like
        Flux array.
    wav_min: float
        Lower bound wavelength value.
    wav_max: float
        Upper bound wavelength value.

    Returns
    -------
    wav_sel: array
        New wavelength array within bounds wav_min, wav_max
    flux_sel: array
        New wavelength array within bounds wav_min, wav_max
        """
    wav = np.asarray(wav, dtype=float)
    flux = np.asarray(flux, dtype=float)

    mask = mask_between(wav, wav_min, wav_max)
    flux_sel = flux[mask]
    wav_sel = wav[mask]

    return wav_sel, flux_sel


def mask_between(x: ndarray, xmin: float, xmax: float) -> ndarray:
    """Create boolean mask of x between xmin and xmax."""
    return (x >= xmin) & (x < xmax)


def silent_remove(filename: str) -> None:
    """Remove file without failing when it doesn't exist."""
    try:
        os.remove(filename)
    except OSError as e:
        if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
            raise  # re-raise exception if a different error occurred


####################################################
def issequenceforme(obj):
    if isinstance(obj, str):
        return False
    return isinstance(obj, collections.Sequence)


def resolutions2ints(resolution: Sequence[Any]) -> List[int]:
    """List of resolutions to list of integer resolutions.

    Convert from ["100k", "30000"] to [100000, 30000].
    """
    if not issequenceforme(resolution):
        raise TypeError(
            "resolution was a {} but needs to be a Sequence".format(type(resolution))
        )

    res_list = []
    for res in resolution:
        res_list.append(res2int(res))
    return res_list


def res2int(res: Any) -> int:
    """Convert from "100k" or "100000" to 100000."""
    if issequenceforme(res):
        raise TypeError("res was a {} but needs to be a non-Sequence".format(type(res)))

    if isinstance(res, (np.int, np.float, int, float)):
        value = res
    elif isinstance(res, str):
        if res.lower().endswith("k"):
            value = float(res[:-1]) * 1000
        else:
            value = float(res)
    else:
        raise TypeError("Resolution name Type error of type {}".format(type(res)))

    return int(value)


def resolutions2strs(resolution: Sequence[Any]) -> List[str]:
    """List of resolutions to list of string resolutions.

    Convert from ["100000", 10000] to ["100k", "10k"].
    """
    if not issequenceforme(resolution):
        raise TypeError(
            "resolution was a {} but needs to be a Sequence".format(type(resolution))
        )

    res_list = []
    for res in resolution:
        res_list.append(res2str(res))
    return res_list


def res2str(res: Any) -> str:
    """Convert from "100000" or 100000 to "100k"."""
    if issequenceforme(res):
        raise TypeError(
            "resolution was a {} but needs to be a non-Sequence".format(type(res))
        )

    if isinstance(res, (np.int, np.float)):
        value = res / 1000
    elif isinstance(res, str):
        if res.lower().endswith("k"):
            value = res[:-1]
        else:
            value = float(res) / 1000
    else:
        raise TypeError("Resolution name TypeError of type {}".format(type(res)))

    return "{0}k".format(int(value))


#################################
def rv_cumulative(rv_vector: Union[List, ndarray], single: bool = False) -> List[float]:
    """Function that calculates the cumulative RV vector weighted_error."""
    if single:
        # Include 1st value for reference
        return [
            weighted_error(rv_vector[0]),
            weighted_error(rv_vector[:2]),
            weighted_error(rv_vector[:3]),
            weighted_error(rv_vector[:4]),
            weighted_error(rv_vector),
        ]

    else:
        return [
            weighted_error(rv_vector[:2]),
            weighted_error(rv_vector[:3]),
            weighted_error(rv_vector[:4]),
            weighted_error(rv_vector),
        ]


def rv_cumulative_full(rv_vector: Union[List, ndarray]) -> ndarray:
    """Function that calculates the cumulative RV vector weighted_error. In both directions."""
    assert len(rv_vector) == 5, "This hardcoded solution only meant for 5 bands."

    cumulation = np.asarray(
        [
            weighted_error(rv_vector[0]),  # First
            weighted_error(rv_vector[:2]),
            weighted_error(rv_vector[:3]),
            weighted_error(rv_vector[:4]),
            weighted_error(rv_vector[:]),  # All
            weighted_error(rv_vector[1:]),
            weighted_error(rv_vector[2:]),
            weighted_error(rv_vector[3:]),
            weighted_error(rv_vector[4]),  # Last
        ],
        dtype=float,
    )
    return cumulation


def weighted_error(rv_vector: Union[List[float], ndarray]) -> float:
    """Function that calculates the average weighted error from a vector of errors."""
    rv_vector = np.asarray(rv_vector)
    rv_value = 1.0 / (np.sqrt(np.sum((1.0 / rv_vector) ** 2.0)))

    return rv_value


def moving_average(x: ndarray, window_size: Union[int, float]) -> ndarray:
    """Moving average."""
    window = np.ones(int(window_size)) / float(window_size)
    return np.convolve(x, window, "same")


#################################
def load_aces_spectrum(
    params: Union[ndarray, List[float]],
    photons: bool = True,
    air: bool = False,
    wl_range: List[float] = [3000, 54000],
):
    """Load a Phoenix spectrum from the phoenix library using STARFISH.

    Parameters
    ----------
    params: ndarray
         [temp, logg, metallicity(, alpha)]
    photons: bool
        Necessary conversions into photons for precisions.
    air: bool
       Convert to air wavelengths (default = False).
    wl_range: [float, float]
        Min/Max wavelength range to load. Default = [3000, 54000] Angstrom.

    Returns
    -------
    wav_micron: ndarray
        Wavelength in microns
    flux_micron: ndarray
        Photon counts per (cm**2 s) or SED/micron (within a multiplicative constant 1/(h*c)).

    Spectra available from http://phoenix.astro.physik.uni-goettingen.de
    """
    base = eniric.paths["phoenix_raw"] + os.sep

    if len(params) == 3:  # Only 3 parameters given
        params = [params[0], params[1], params[2], 0]  # Set alpha=0

    if len(params) == 4:
        from Starfish.grid_tools import PHOENIXGridInterface as PHOENIX

        phoenix_grid = PHOENIX(base=base, air=air, norm=False, wl_range=wl_range)

    else:
        raise ValueError("Number of parameters is incorrect")

    wav = phoenix_grid.wl
    flux, hdr = phoenix_grid.load_flux(params)

    # Convert wavelength Angstrom to micron
    wav_micron = wav * 10 ** -4
    # Convert SED from /cm  to /micron
    flux_micron = flux * 10 ** -4

    if photons:
        # Convert to photons
        # The energy units of Phoenix fits files is erg/s/cm**2/cm
        # PHOENIX ACES gives the Spectral Energy Density (SED)
        # We transform the SED into photons by
        # multiplying the flux by the wavelength (lambda)
        #
        #     Flux_photon = Flux_energy/Energy_photon
        # with
        #     Energy_photon = h*c/lambda
        # Flux_photon = Flux_energy * lambda / (h * c)

        flux_micron = flux_micron * wav_micron
    return wav_micron, flux_micron


def load_btsettl_spectrum(
    params: Union[ndarray, List[float]],
    photons: bool = True,
    air: bool = False,
    wl_range: List[float] = [3000, 30000],
):
    """Load a BT-Settl spectrum from the CIFIST2011 library using STARFISH.

    Parameters
    ----------
    params: ndarray
         [temp, logg]. Metallicity = 0, alpha = 0
    photons: bool
        Necessary conversions into photons for precisions.
    air: bool
       Convert to air wavelengths (default = False).
    wl_range: [float, float]
        Min/Max wavelength range to load. Default = [3000, 30000] Angstrom.

    Returns
    -------
    wav_micron: ndarray
        Wavelength in microns
    flux_micron: ndarray
        Photon counts per (cm**2 s) or SED/micron. (within a multiplicative constant 1/(h*c))

    Notes
    -----
    From BT-SETTL readme:
        CIFIST2011_2015: published version of the BT-Settl grid (Baraffe et al. 2015,
        Allard et al. 2015. This grid will be the most complete
        of the CIFIST2011 grids above, but currently: Teff = 1200 - 7000K, logg = 2.5 - 5.5,
        [M/H] = 0.0.

        Available from https://phoenix.ens-lyon.fr/Grids/BT-Settl/CIFIST2011_2015/FITS/
    """
    from Starfish.grid_tools import CIFISTGridInterface as BTSETTL

    if (2 < len(params)) and (len(params) <= 4):
        assert params[2] == 0
        assert params[-1] == 0  # Checks index 3 when present.
        params = params[0:2]  # Only allow 2 params

    base = eniric.paths["btsettl_raw"] + os.sep

    btsettl_grid = BTSETTL(base=base, air=air, norm=False, wl_range=wl_range)

    wav = btsettl_grid.wl
    # Convert wavelength from Angstrom to micron
    wav_micron = wav * 10 ** -4

    # CIFIST flux is  W/m**2/um
    flux, hdr = btsettl_grid.load_flux(params)

    flux_micron = flux * 10 ** -7  # Convert W/m**2/um to ergs/s/m**2/um)
    flux_micron *= 10 ** -4  # Convert 1/m**2 to 1/cm**2

    if photons:
        # Convert to photon counts:
        # The energy units of CIFIST fits files is W/m**2/um
        # We have converted it to ergs/(s cm**2 um)
        # BT-Settl gives the Spectral Energy Density (SED)
        # We transform the SED into photons by
        # multiplying the flux by the wavelength (lambda)
        #
        #     Flux_photon = Flux_energy/Energy_photon
        # with
        #     Energy_photon = h*c/lambda
        # Flux_photon = Flux_energy * lambda / (h * c)
        flux_micron = flux_micron * wav_micron
    return wav_micron, flux_micron


#####################################################
def doppler_shift_wav(wavelength: ndarray, vel: float):
    r"""Doppler shift wavelength by a given velocity (non-relativistic).

    Apply Doppler shift to the wavelength values of the spectrum
    using the velocity value provided and the relation
    \(\Delta\lambda / \lambda = v / c\)

    Parameters
    ----------
    wavelength: ndarray
        Wavelength vector
    vel : float
        Velocity to Doppler shift by in km/s.

    Notes
    -----
    The Doppler shift is calculated using the relation
       \Delta\lambda / \lambda\] = \[v / c
    Where RV is the radial velocity (in km/s), \(\lambda_0\)`
    is the rest wavelength and \(\Delta\lambda\) is the wavelength
    shift, \(\lambda_{shift} - \lambda_0\)
    """

    if not np.isfinite(vel):
        ValueError("The velocity is not finite.")

    shifted_wavelength = wavelength * (1 + (vel * 1000 / c.value))
    return shifted_wavelength


def doppler_shift_flux(
    wavelength: ndarray, flux: ndarray, vel: float, new_wav: Optional[ndarray] = None
):
    r"""Doppler shift flux by a given velocity, return it at the original wavelengths (non-relativistic).

    Apply Doppler shift to the wavelength values of the spectrum
    using the velocity value provided and the relation
    \(\Delta\lambda / \lambda = v / c\)

    Then linearly interpolate the flux with the new wavelength to the old wavelengths.

    Parameters
    ----------
    wavelength: ndarray
        Wavelength vector
    flux: ndarray
        Flux vector
    vel : float
        Velocity to Doppler shift by in km/s.
    new_wav: Optional[ndarray]
        New wavelength array to evaluate the doppler shifted flux at.
        If None then defaults to new_wav=wavelength.
    Returns
    -------
    new_flux: ndarray
        Doppler-shifted flux evaluated at new_wav.
    """
    shifted_wavelength = doppler_shift_wav(wavelength, vel)

    if new_wav is None:
        new_wav = wavelength
    new_flux = np.interp(new_wav, shifted_wavelength, flux)
    return new_flux


def doppler_limits(rvmax, wmin, wmax):
    """Calculate wavelength limits to apply if preforming doppler shift.

    To avoid any edge effects within wmin and wmax after doppler shift.

    Parameters
    ----------
    rvmax: float
        Maximium absolute RV offset in km/s. Uses np.abs() to constrain as absolute.
    wmin: float
        Lower wavelength limit.
    wmax: float
        Upper wavelength limit.
    Returns
    -------
    new_wmin: float
      Lower wavelength bound shifted by -rvmax
    new_wmax: float
       Lower wavelength bound shifted by +rvmax
    """
    c_km = const.c.value / 1000  # c in km/s
    doppler_minus, doppler_plus = (1 - np.abs(rvmax) / c_km), (1 + np.abs(rvmax) / c_km)

    return wmin * doppler_minus, wmax * doppler_plus



"""Plot a Figueira et al. 2016 like plot."""

import os
import sys

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

import eniric
from eniric.utilities import rv_cumulative, rv_cumulative_full

matplotlib.use("Agg")


def load_dataframe(filename):
    """Load in precision file, clean up spaces in csv."""
    df = pd.read_csv(precision_file)
    # Temp, logg, [Fe/H], Alpha, Band, Resolution, vsini, Sampling, Quality, Cond. 1, Cond. 2, Cond. 3, correct flag
    df.columns = df.columns.str.strip()
    df.Band = df.Band.str.strip()
    df.Resolution = df.Resolution.str.strip()
    df.Quality = df.Quality.astype(float)
    df.Temp = df.Temp.astype(float)
    df.logg = df.logg.astype(float)
    df["[Fe/H]"] = df["[Fe/H]"].astype(float)
    df.Alhpa = df.Alpha.astype(float)
    df.vsini = df.vsini.astype(float)
    df.Sampling = df.Sampling.astype(float)
    df["Cond. 1"] = df["Cond. 1"].astype(float)
    df["Cond. 2"] = df["Cond. 2"].astype(float)
    df["Cond. 3"] = df["Cond. 3"].astype(float)
    df["correct flag"] = df["correct flag"].astype(bool)

    return df


def plot_precision(
    precision_file, teffs=None, logg=4.5, fe_h=0.0, vsini=1.0, sampling=3
):
    """Plot precision 4 panel with RV. precision."""
    if teffs is None:
        # Default teffs
        teffs = [3900, 3500, 2800, 2600]
    assert len(teffs) == 4
    df = load_dataframe(precision_file)
    filter_dict = {"logg": logg, "[Fe/H]": fe_h, "vsini": vsini, "Sampling": sampling}
    df = filter_df(
        df, filter_dict, drop_list=["Alpha", "[Fe/H]", "correct flag", "Quality"]
    )

    fig, axes = plt.subplots(2, 2)
    ax = axes.flatten()
    df_selected = df[df.Resolution.isin(["60k", "80k", "100k"])]
    df_selected = df_selected[df_selected.Temp.isin(teffs)]

    maximums = []
    minimums = []

    for ii, temp in enumerate(teffs):
        # This entry
        df_ii = df_selected[df_selected["Temp"] == temp]
        df_ii_60k = df_ii[df_ii["Resolution"].str.strip() == "60k"]
        df_ii_80k = df_ii[df_ii["Resolution"].str.strip() == "80k"]
        df_ii_100k = df_ii[df_ii["Resolution"].str.strip() == "100k"]

        df_ii_60k = df_ii_60k.set_index("Band")
        print("new index\n", df_ii_60k)
        df_ii_60k = df_ii_60k.reindex(["Z", "Y", "J", "H", "K"])
        print("reindexed\n", df_ii_60k)

        df_ii_80k = df_ii_80k.set_index("Band")
        df_ii_80k = df_ii_80k.reindex(["Z", "Y", "J", "H", "K"])
        df_ii_100k = df_ii_100k.set_index("Band")
        df_ii_100k = df_ii_100k.reindex(["Z", "Y", "J", "H", "K"])

        maximums.append(
            np.max(
                [
                    df_ii_60k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                    df_ii_80k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                    df_ii_100k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                ]
            )
        )
        minimums.append(
            np.min(
                [
                    df_ii_60k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                    df_ii_80k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                    df_ii_100k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                ]
            )
        )

        ax[ii].fill_between(
            df_ii_60k.index,
            df_ii_60k["Cond. 2"].values,
            df_ii_60k["Cond. 3"].values,
            color="b",
            alpha=0.2,
        )
        ax[ii].fill_between(
            df_ii_80k.index,
            df_ii_80k["Cond. 2"].values,
            df_ii_80k["Cond. 3"].values,
            color="g",
            alpha=0.2,
        )
        ax[ii].fill_between(
            df_ii_100k.index,
            df_ii_100k["Cond. 2"].values,
            df_ii_100k["Cond. 3"].values,
            color="r",
            alpha=0.2,
        )

        ax[ii].plot(
            df_ii_60k.index, df_ii_60k["Cond. 1"].values, color="b", linestyle="--"
        )  # lim
        ax[ii].plot(
            df_ii_80k.index, df_ii_80k["Cond. 1"].values, color="g", linestyle="--"
        )  # lim
        ax[ii].plot(
            df_ii_100k.index, df_ii_100k["Cond. 1"].values, color="r", linestyle="--"
        )  # lim

        ax[ii].scatter(
            df_ii_60k.index,
            df_ii_60k["Cond. 2"].values,
            marker="^",
            color="b",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_60k.index,
            df_ii_60k["Cond. 3"].values,
            marker="o",
            color="b",
            alpha=0.4,
        )

        ax[ii].scatter(
            df_ii_80k.index,
            df_ii_80k["Cond. 3"].values,
            marker="^",
            color="g",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_80k.index,
            df_ii_80k["Cond. 2"].values,
            marker="o",
            color="g",
            alpha=0.4,
        )

        ax[ii].scatter(
            df_ii_100k.index,
            df_ii_100k["Cond. 3"].values,
            marker="^",
            color="r",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_100k.index,
            df_ii_100k["Cond. 2"].values,
            marker="o",
            color="r",
            alpha=0.4,
        )

    # Set limits ticks and labels
    ymax = np.max(maximums)
    ymin = np.min(minimums)
    delta_y = ymax - ymin
    band_size = len(df_ii_60k.index)
    for jj in range(4):
        ax[jj].text(0, ymax, "{} K".format(teffs[jj]), size=14)
        ax[jj].set_ylim(ymin - 0.11 * delta_y, ymax + 0.11 * delta_y)
        ax[jj].set_xlim(-0.5, band_size - 0.5)
        ax[jj].tick_params(axis="both", which="major", labelsize=12)

        # ticks and labels
        if (jj == 2) or (ii == 3):
            ax[jj].set_xlabel("Bands", fontsize=12)
        if (jj == 1) or (jj == 3):
            ax[jj].set_yticklabels([])
        if (jj == 0) or (jj == 1):
            ax[jj].set_xticklabels([])

    fig.text(
        0.04,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=12,
    )

    fig.subplots_adjust(hspace=0, wspace=0, bottom=0.12, top=0.95, right=0.95)

    fig.savefig("plots/precision_logg{0}_feh_{1}.pdf".format(logg, fe_h))
    fig.savefig("plots/precision_logg{0}_feh_{1}.png".format(logg, fe_h), dpi=400)


def filter_df(df, filter_dict, drop_list=None):
    """Filter DataFrame by dictionary of key and values."""
    for key, val in filter_dict.items():
        df = df[df[key] == val]
    if drop_list is not None:
        df = df.drop(columns=drop_list)
    return df


def cumulative_df(df, full_cum=False):
    """Index by band, """
    bands = df.index
    assert all(bands == ["Z", "Y", "J", "H", "K"]), bands
    if full_cum:
        cum_bands = ["Z", "ZY", "ZYJ", "ZYJH", "ZYJHK", "YJHK", "JHK", "HK", "K"]

        cum_dict = {
            "Band": cum_bands,
            "Cond. 1": rv_cumulative_full(df["Cond. 1"]),
            "Cond. 2": rv_cumulative_full(df["Cond. 2"]),
            "Cond. 3": rv_cumulative_full(df["Cond. 3"]),
        }
    else:
        cum_bands = ["Z", "ZY", "ZYJ", "ZYJH", "ZYJHK"]
        cum_dict = {
            "Band": cum_bands,
            "Cond. 1": rv_cumulative(df["Cond. 1"], single=True),
            "Cond. 2": rv_cumulative(df["Cond. 2"], single=True),
            "Cond. 3": rv_cumulative(df["Cond. 3"], single=True),
        }
    cum_df = pd.DataFrame(cum_dict)
    cum_df = cum_df.set_index("Band")
    cum_df = cum_df.reindex(cum_bands)
    return cum_df


def cumulative_plot(
    precision_file,
    teffs=None,
    logg=4.5,
    fe_h=0.0,
    vsini=1.0,
    sampling=3,
    full_cum=False,
):
    """RV precision with cumulative bands.

    full_cum: bool
    Cumlative over entire range [ "Z","ZY", "ZYJ", "ZYJH", "ZYJHK","YJHK", "JHK","HK","K"]

    """
    if teffs is None:
        # Default values
        teffs = [3900, 3500, 2800, 2600]
    assert len(teffs) == 4

    df = load_dataframe(precision_file)
    filter_dict = {"logg": logg, "[Fe/H]": fe_h, "vsini": vsini, "Sampling": sampling}
    df = filter_df(
        df, filter_dict, drop_list=["Alpha", "[Fe/H]", "correct flag", "Quality"]
    )

    fig, axes = plt.subplots(2, 2)
    ax = axes.flatten()
    df_selected = df[df.Resolution.isin(["60k", "80k", "100k"])]
    df_selected = df_selected[df_selected.Temp.isin(teffs)]

    maximums = []
    minimums = []

    for ii, temp in enumerate(teffs):
        # This entry
        df_ii = df_selected[df_selected["Temp"] == temp]
        df_ii_60k = df_ii[df_ii["Resolution"].str.strip() == "60k"]
        df_ii_80k = df_ii[df_ii["Resolution"].str.strip() == "80k"]
        df_ii_100k = df_ii[df_ii["Resolution"].str.strip() == "100k"]

        df_ii_60k = df_ii_60k.set_index("Band")
        print("new index\n", df_ii_60k)
        df_ii_60k = df_ii_60k.reindex(["Z", "Y", "J", "H", "K"])
        print("reindexed\n", df_ii_60k)

        df_ii_80k = df_ii_80k.set_index("Band")
        df_ii_80k = df_ii_80k.reindex(["Z", "Y", "J", "H", "K"])
        df_ii_100k = df_ii_100k.set_index("Band")
        df_ii_100k = df_ii_100k.reindex(["Z", "Y", "J", "H", "K"])

        # Cumulative
        df_ii_60k = cumulative_df(df_ii_60k, full_cum=full_cum)
        df_ii_80k = cumulative_df(df_ii_80k, full_cum=full_cum)
        df_ii_100k = cumulative_df(df_ii_100k, full_cum=full_cum)

        maximums.append(
            np.max(
                [
                    df_ii_60k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                    df_ii_80k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                    df_ii_100k[["Cond. 1", "Cond. 2", "Cond. 3"]].max(),
                ]
            )
        )
        minimums.append(
            np.min(
                [
                    df_ii_60k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                    df_ii_80k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                    df_ii_100k[["Cond. 1", "Cond. 2", "Cond. 3"]].min(),
                ]
            )
        )

        ax[ii].fill_between(
            df_ii_60k.index,
            df_ii_60k["Cond. 2"].values,
            df_ii_60k["Cond. 3"].values,
            color="b",
            alpha=0.2,
        )
        ax[ii].fill_between(
            df_ii_80k.index,
            df_ii_80k["Cond. 2"].values,
            df_ii_80k["Cond. 3"].values,
            color="g",
            alpha=0.2,
        )
        ax[ii].fill_between(
            df_ii_100k.index,
            df_ii_100k["Cond. 2"].values,
            df_ii_100k["Cond. 3"].values,
            color="r",
            alpha=0.2,
        )

        ax[ii].plot(
            df_ii_60k.index, df_ii_60k["Cond. 1"].values, color="b", linestyle="--"
        )  # lim
        ax[ii].plot(
            df_ii_80k.index, df_ii_80k["Cond. 1"].values, color="g", linestyle="--"
        )  # lim
        ax[ii].plot(
            df_ii_100k.index, df_ii_100k["Cond. 1"].values, color="r", linestyle="--"
        )  # lim

        ax[ii].scatter(
            df_ii_60k.index,
            df_ii_60k["Cond. 2"].values,
            marker="^",
            color="b",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_60k.index,
            df_ii_60k["Cond. 3"].values,
            marker="o",
            color="b",
            alpha=0.4,
        )

        ax[ii].scatter(
            df_ii_80k.index,
            df_ii_80k["Cond. 3"].values,
            marker="^",
            color="g",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_80k.index,
            df_ii_80k["Cond. 2"].values,
            marker="o",
            color="g",
            alpha=0.4,
        )

        ax[ii].scatter(
            df_ii_100k.index,
            df_ii_100k["Cond. 3"].values,
            marker="^",
            color="r",
            alpha=0.4,
        )
        ax[ii].scatter(
            df_ii_100k.index,
            df_ii_100k["Cond. 2"].values,
            marker="o",
            color="r",
            alpha=0.4,
        )

        ax[ii].fill_between(
            df_ii_100k.index,
            df_ii_100k["Cond. 2"].values,
            df_ii_100k["Cond. 3"].values,
            color="r",
            alpha=0.2,
        )

    # Set limits ticks and labels
    ymax = np.min([np.max(maximums), 10])  # Set limit to 20 km/s
    ymin = np.min(minimums)
    delta_y = ymax - ymin
    band_size = len(df_ii_60k.index)

    for jj in range(4):
        ax[jj].text(0, ymax, "{} K".format(teffs[jj]), size=14)
        ax[jj].set_ylim(ymin - 0.1 * delta_y, ymax + 0.15 * delta_y)
        ax[jj].set_xlim(-0.5, band_size - 0.5)
        ax[jj].tick_params(axis="both", which="major", labelsize=12)

        # ticks and labels
        if (jj == 2) or (ii == 3):
            ax[jj].set_xlabel("Bands", fontsize=12)
            if full_cum:
                ax[jj].tick_params(axis="x", labelrotation=40)
            else:
                ax[jj].tick_params(axis="x", labelrotation=25)

        if (jj == 1) or (jj == 3):
            ax[jj].set_yticklabels([])
        if (jj == 0) or (jj == 1):
            ax[jj].set_xticklabels([])

    fig.text(
        0.04,
        0.5,
        r"Precision [m/s]",
        ha="center",
        va="center",
        rotation="vertical",
        size=12,
    )

    fig.subplots_adjust(hspace=0, wspace=0, bottom=0.17, top=0.95, right=0.95)

    fname = "plots/cummulative_precision_logg{0}_feh_{1}_{2}".format(
        logg, fe_h, full_cum
    )
    fig.savefig(fname + ".pdf")
    fig.savefig(fname + ".png", dpi=400)


if __name__ == "__main__":
    # ToDo: Add an argparse cli

    precision_file = os.path.join(eniric.paths["precision"], "precision_results.csv")
    plot_precision(precision_file, teffs=[3900, 3500, 2800, 2600])
    # plot_precision(precision_file, teffs=[3900, 3500, 2800, 2600], logg=4, fe_h=1)

    cumulative_plot(precision_file, teffs=[3900, 3500, 2800, 2600])
    cumulative_plot(precision_file, teffs=[3900, 3500, 2800, 2600], full_cum=True)
    sys.exit(0)



"""Functions to read column-separated files.

These are a wrapper around pandas.
"""
from typing import List, Optional, Tuple

import pandas as pd
from numpy import ndarray


# noinspection SpellCheckingInspection,SpellCheckingInspection
def pdread_2col(filename: str, noheader: bool = False) -> Tuple[ndarray, ndarray]:
    """Read in a 2 column file with pandas.

    Parameters
    ----------
    filename: str
        Name of file to read.
    noheader: bool
        Flag indicating if there is no column names given in file.
        Default = False.

    Returns
    -------
    col1: ndarray
        First column as float.
    col2: ndarray
        Second column as float.
    """
    try:
        if noheader:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2"],
                header=None,
                dtype=float,
                delim_whitespace=True,
            )
        else:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2"],
                dtype=float,
                delim_whitespace=True,
            )
    except Exception as e:
        print("There was an error trying to read in the file \n{}".format(filename))
        raise e

    return data["col1"].values, data["col2"].values


def pdread_3col(
    filename: str, noheader: bool = False
) -> Tuple[ndarray, ndarray, ndarray]:
    """Read in a 3 column file with pandas.

    Parameters
    ----------
    filename: str
        Name of file to read.
    noheader: bool
        Flag indicating if there is no column names given in file

    Returns
    -------
    col1: ndarray
        First column as float.
    col2: ndarray
        Second column as float.
    col3: ndarray
        Third column as float.
    """
    try:
        if noheader:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2", "col3"],
                header=None,
                dtype=float,
                delim_whitespace=True,
            )
        else:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2", "col3"],
                dtype=float,
                delim_whitespace=True,
            )
    except Exception as e:
        print("There was an error trying to read in the file \n{}".format(filename))
        raise e

    return data["col1"].values, data["col2"].values, data["col3"].values


def pdread_4col(
    filename: str, noheader: bool = False
) -> Tuple[ndarray, ndarray, ndarray, ndarray]:
    """Read in a 4 column file with pandas.

    Parameters
    ----------
    filename: str
        Name of file to read.
    noheader: bool
        Flag indicating if there is no column names given in file

    Returns
    -------
    col1: ndarray
        First column as float.
    col2: ndarray
        Second column as float.
    col3: ndarray
        Third column as float.
    col4: ndarray
        Fourth column as float.
    """
    try:
        if noheader:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2", "col3", "col4"],
                header=None,
                dtype=float,
                delim_whitespace=True,
            )
        else:
            data = pd.read_table(
                filename,
                comment="#",
                names=["col1", "col2", "col3", "col4"],
                dtype=float,
                delim_whitespace=True,
            )
    except Exception as e:
        print("There was an error trying to read in the file \n{}".format(filename))
        raise e

    return (
        data["col1"].values,
        data["col2"].values,
        data["col3"].values,
        data["col4"].values,
    )


def pdwrite_2col(
    filename: str,
    data1: ndarray,
    data2: ndarray,
    sep: str = "\t",
    header: Optional[List[str]] = None,
    float_format: Optional[str] = None,
) -> int:
    """Write out a 2 column file with pandas.

    Parameters
    ----------
    filename: str
        Name of file to write.
    data1: ndarray or list, array-like
        The data for the first column
    data2: ndarray or list, array-like
        The data for the second column
    sep: str
        Character separation between values.
    header: Optional list of strings
        Header strings to apply to columns.
    float_format: str default None
        Specify floating point string format.

    Returns
    -------
    flag: bool
        Returns 0 if successful.
    """
    if header is not None:
        df = pd.DataFrame({"# {}".format(header[0]): data1, header[1]: data2})
    else:
        df = pd.DataFrame({"# x": data1, "y": data2})

    # Write DataFrame to file
    df.to_csv(
        filename, sep=sep, header=header, index=False, float_format=float_format
    )  # header=False

    return 0


def pdwrite_3col(
    filename: str,
    data1: ndarray,
    data2: ndarray,
    data3: ndarray,
    sep: str = "\t",
    header: Optional[List[str]] = None,
    float_format: Optional[str] = None,
) -> int:
    """Write out a 3 column file with pandas.

    Parameters
    ----------
    filename: str
        Name of file to write.
    data1: ndarray or list, array-like
        The data for the first column
    data2: ndarray or list, array-like
        The data for the second column
    data3: ndarray or list, array-like
        The data for the third column
    sep: str
        Character separation between values.
    header: optional list of strings
        Header strings to apply to columns.
    float_format: str default None
        Specify floating point string format.

    Returns
    -------
    flag: bool
        Returns 0 if successful.
    """
    if header is not None:
        df = pd.DataFrame(
            {"# {}".format(header[0]): data1, header[1]: data2, header[2]: data3}
        )
    else:
        df = pd.DataFrame({"# x": data1, "y": data2, "z": data3})

    # Write DataFrame to file
    df.to_csv(
        filename, sep=sep, header=header, index=False, float_format=float_format
    )  # header=False

    return 0


def pdwrite_cols(filename: str, *data, **kwargs) -> int:
    """Write out a csv file with pandas, variable columns possible.

    Parameters
    ----------
    filename: str
        Name of file to write.
    *data: ndarray or list, array-like
        Variable number of data columns to be writen in the given order.
    **kwargs: dict
        Keyword args for pandas
    sep: str, default="\t"
        Character separation between values.
    header: optional list of strings or bool
        Header strings to apply to columns. Must be equal to number
        of data columns provided.

    Returns
    -------
    flag: bool
        Returns 0 if successful.
    """

    # unpack keyword args, second argument is the default if not found.
    header = kwargs.pop("header", None)
    sep = kwargs.pop("sep", "\t")
    index = kwargs.pop("index", False)
    float_format = kwargs.pop("float_format", "%.6f")
    # TODO: See about passing any extra keywords into pandas call
    if kwargs:  # check for unwanted key words
        raise TypeError("Unexpected **kwargs: {!r}".format(kwargs))

    if header is not None:
        if len(header) != len(data):
            raise ValueError("Size of data and header does not match.")

    data_dict = {}
    for i, data_i in enumerate(data):
        data_dict[i] = data_i  # keys are assigned the index value from enumerate

        if len(data[i]) != len(data[0]):
            raise ValueError("The length of the data columns are not equal")

    df = pd.DataFrame(data_dict)

    write_sequence = range(len(data))  # key values to write data in order

    # Write DataFrame to file
    df.to_csv(
        filename,
        columns=write_sequence,
        sep=sep,
        header=header,
        index=index,
        float_format=float_format,
    )

    return 0



#!/usr/bin/env python
"""Script to process weekly TAPAS spectra to form an average atmospheric spectrum.

"""
import os
import string

import numpy as np

import eniric.io_module as io

dirmodels = "/home/pfigueira/data/tapas/nIRanalysis_visible/"
list_files = "list_tapas_models.txt"

outdir = "atmmodel/"


def read_tapas(filename):
    """Read in tapas file."""
    file_conv = io.read_fullcol(filename)
    properties_dict = {}
    lambdas = []
    flux = []
    for line in file_conv:
        if line[0] == "|":
            # this is to ignore the header of the plot
            continue
        elif line[0] == "\\":
            # add everything starting with a \\ to the dictionary
            # remove the "\" form the beginning and the "\n" from the end
            key_dic, value_dic = string.split(line[1:-1], "=")
            properties_dict[key_dic] = value_dic
        else:
            # these are the values of the file itself
            lambda_act, flux_act = string.split(line)
            lambdas.append(float(lambda_act))
            flux.append(float(flux_act))

    lambdas = np.array(lambdas)
    flux = np.array(flux)
    # note that the lambdas are provided in reversed order
    return [properties_dict, lambdas[::-1], flux[::-1]]


def read_allfiles(mask_limit=0.02):
    """Read all the files in list_files."""
    files = io.read_fullcol(os.path.join(dirmodels, list_files))
    print("Reading the files...")
    atm_models = [
        read_tapas(os.path.join(dirmodels, dirmodels + file_act[:-1]))
        for file_act in files
    ]
    print("done.")

    wav = atm_models[0][1]
    mean_flux = []
    std_flux = []
    mask = []
    for i, __ in enumerate(wav):
        flux_at_wav = [model[2][i] for model in atm_models]
        mean_flux.append(np.average(flux_at_wav))
        std_flux.append(np.std(flux_at_wav))
        if mean_flux[-1] > (1.0 - mask_limit):
            # if transmission above threshold do not mask, otherwise mask
            mask.append(1.0)
        else:
            mask.append(0.0)

    write_4col_eeed(
        os.path.join(outdir, "{0}_visible.txt".format(eniric.atmmodel["base"])),
        wav,
        mean_flux,
        std_flux,
        mask,
    )


def write_4col_eeed(filename, data1, data2, data3, data4):
    """Write data in 3 columns separated by tabs in a "filename" file."""
    f = open(filename, "w")

    for i, __ in enumerate(data1):
        # f.write("\t"+str(data1[i])+"\t\t"+str(data2[i])+"\t\t"+str(data3[i])+"\n")
        f.write(
            "\t{0:e}\t\t{1:e}\t\t{2:e}\t\t{3:d}\n".format(
                data1[i], data2[i], data3[i], data4[i]
            )
        )

    f.close()



#!/usr/bin/env python
# Script to perform a convolution on a spectrum.
# Can take a number of parameters if needed
import argparse
import os
import sys
from datetime import datetime as dt
from typing import List, Optional, Sequence, Union

import eniric
from eniric.obsolete.nIRanalysis import convolve_spectra
from eniric.obsolete.utilities import get_spectrum_name, resampler
from eniric.utilities import resolutions2ints


def _parser():
    """Take care of all the argparse stuff.

    :returns: the args
    """
    parser = argparse.ArgumentParser(
        description="Perform Convolution in preparation for nIR_precision."
    )
    parser.add_argument(
        "-s", "--startype", help='Spectral Type e.g "MO"', type=str, nargs="+"
    )
    parser.add_argument(
        "-v", "--vsini", help="Rotational velocity of source", type=float, nargs="+"
    )
    parser.add_argument(
        "-R",
        "--resolution",
        help="Observational resolution e.g. 100000 or 100k",
        type=str,
        nargs="+",
    )
    parser.add_argument(
        "-b",
        "--band",
        type=str,
        default="ALL",
        choices=eniric.bands["all"],
        help="Wavelength band to select",
        nargs="+",
    )
    parser.add_argument(
        "--sample_rate",
        default=[3.0],
        type=float,
        nargs="+",
        help="Resample rate, pixels per FWHM. Default=3.0",
    )
    parser.add_argument(
        "--noresample", help="Don't Resample output", default=False, action="store_true"
    )
    parser.add_argument(
        "--unnormalized", help="Normalize for wavelength step", action="store_true"
    )
    parser.add_argument(
        "--org",
        help="Only use original .dat files, (temporary option)",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "-r",
        "--replace",
        action="store_true",
        help="Replace data files if already created.",
    )
    return parser.parse_args()


def main(
    startype: Sequence[str],
    vsini: Sequence[float],
    resolution: Sequence[str],
    band: Sequence[str],
    sample_rate: Optional[List[Union[int, float]]] = None,
    noresample: bool = False,
    unnormalized: bool = False,
    org: bool = False,
    replace: bool = False,
):
    """Run convolutions of NIR spectra for the range of given parameters.

    Multiple values of startype, vsini, resolution, band, and sample_rate can
    be provided.

    Read files from eniric.paths["phoenix_dat"]"

    Parameters
    ----------
    startype: list of strings
    vsini: list of floats
    resolution: list of floats
    band: list of strings
    sample_rate: list of floats default=[3.0]
    noresample: bool default=False
    unnormalized: bool default=False

    """
    normalize = not unnormalized
    if sample_rate is None:
        # Default sample rate
        sample_rate = [3.0]

    # Check the inputs are correct format. (lists)
    for f_input, f_name in zip(
        [startype, band, vsini, resolution, sample_rate],
        ["startype", "band", "vsini", "resolution", "sample_rate"],
    ):
        if not isinstance(f_input, list):
            print(f_name, type(f_input), type(f_name))
            raise TypeError("Input {0} is not list".format(f_name))

    # vsini, resolution, band and sample_rate can all be a series of values
    vsini = [float(v) for v in vsini]  # turn to floats

    # Handle K in Resolution
    res = resolutions2ints(resolution)

    start_time = dt.now()

    phoenix_path = eniric.paths["phoenix_dat"]

    results_dir = eniric.paths["results"]
    os.makedirs(results_dir, exist_ok=True)

    resampled_dir = eniric.paths["resampled"]
    os.makedirs(resampled_dir, exist_ok=True)

    if not normalize:
        norm_ = "_unnormalized"
    else:
        norm_ = ""

    counter = 0
    for star in startype:
        spectrum_name = os.path.join(phoenix_path, get_spectrum_name(star, org=org))

        for b in band:
            for vel in vsini:
                for R in res:
                    for sample in sample_rate:
                        r = int(float(R) / 1000)
                        result_name = "Spectrum_{0}-PHOENIX-ACES_{1}band_vsini{2:.01f}_R{3}k{4}.txt".format(
                            star, b, vel, r, norm_
                        )

                        if (
                            os.path.exists(os.path.join(results_dir, result_name))
                            and not replace
                        ):
                            print(
                                "Skipping convolution as {} already exists".format(
                                    result_name
                                )
                            )
                            continue

                        print("Name to be the result file", result_name)
                        convolve_spectra(
                            spectrum_name,
                            b,
                            vel,
                            R,
                            epsilon=0.6,
                            fwhm_lim=5.0,
                            num_procs=None,
                            results_dir=results_dir,
                            normalize=normalize,
                            output_name=result_name,
                        )

                        # Resample only the file just made
                        if not noresample:
                            resampler(
                                result_name,
                                results_dir=results_dir,
                                resampled_dir=resampled_dir,
                                sampling=sample,
                            )
                        counter += 1

    print(
        "Time to convolve {0:d} combinations = {1}".format(
            counter, dt.now() - start_time
        )
    )
    return 0


if __name__ == "__main__":
    args = vars(_parser())
    opts = {k: args[k] for k in args}

    sys.exit(main(**opts))



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with codecs.open(os.path.join(here, "README.md")) as f:
    long_description = f.read()

config = {
    "description": "Extended NIR Information Content",
    "long_description": long_description,
    "long_description_content_type": "text/markdown",
    "author": "Jason Neal",
    "url": "https://github.com/jason-neal/eniric.git",
    "download_url": "https://github.com/jason-neal/eniric.git",
    "author_email": "jason.neal@astro.up.pt",
    "version": "1.0rc1",
    "license": "MIT",
    "setup_requires": ["pytest-runner"],
    "install_requires": [
        "astropy",
        "joblib>0.12",
        "matplotlib",
        "multiprocess",
        "numpy",
        "pandas",
        "pyyaml",
        "scipy",
        "tqdm",
    ],
    "extras_require": {
        "dev": ["check-manifest"],
        "test": ["coverage", "pytest", "pytest-cov", "python-coveralls", "hypothesis"],
    },  # $ pip install -e .[dev,test]
    "packages": ["eniric", "eniric_scripts", "eniric.obsolete"],
    "scripts": [
        "eniric_scripts/phoenix_precision.py",
        "eniric_scripts/unzip_testdata.py",
        "eniric_scripts/split_atmmodel.py",
        "eniric_scripts/bary_shift_atmmodel.py",
        "eniric/obsolete/make_test_data.py",
        "eniric/obsolete/nIR_run.py",
        "eniric/obsolete/nIR_precision.py",
        "eniric/obsolete/prepare_data.py",
    ],
    "name": "eniric",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    "classifiers": [
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Astronomy",
        "Topic :: Scientific/Engineering :: Physics",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
    ],
    # What does your project relate to?
    "keywords": [
        "Astronomy",
        "Radial velocity",
        "Near-infrared spectroscopy",
        "M-dwarfs",
    ],
}

setup(**config)
